//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionMetadata.h>

@class PTDiffusionStringFieldMetadata;
@class PTDiffusionNumberFieldMetadata;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief Field metadata describes a data item that can have a value.
 
 A field metadata object is immutable.
 
 Default multiplicity is defined by the `defaltMultiplicity` class method on PTDiffusionMultiplicity.
 
 @see PTDiffusionRecordMetadata

 @since 5.6
 */
@interface PTDiffusionFieldMetadata : PTDiffusionMetadata

/**
 Returns default field metadata, representing a string field with unbounded multiplicity.

 @since 5.7
 */
+(PTDiffusionStringFieldMetadata *)defaultMetadata;

-(instancetype)init NS_UNAVAILABLE;

/**
 The class of object produced when transforming field data using the receiver.

 @since 5.6
 */
@property(nonatomic, readonly) Class<NSObject, NSCopying, NSSecureCoding> valueClass;

/**
 Compares the receiver to the given field metadata.

 @param fieldMetadata The field metadata object with which to compare the receiver.

 @return `YES` if the field metadata is equal to the receiver, otherwise `NO`.

 @since 5.6
 */
-(BOOL)isEqualToFieldMetadata:(nullable PTDiffusionFieldMetadata *)fieldMetadata;

/**
 Returns a string field metadata object initialized with the given name and default multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @exception NSInvalidArgumentException If the given name is `nil`.

 @since 5.6
 */
+(PTDiffusionStringFieldMetadata *)stringWithName:(NSString *)name;

/**
 Returns a string field metadata object initialized with the given name and multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @param multiplicity The multiplicity to use. It governs how many fields
 can be transformed using the receiver from the enclosing record.

 @exception NSInvalidArgumentException If either the given name or multiplicity is `nil`.

 @since 5.6
 */
+(PTDiffusionStringFieldMetadata *)stringWithName:(NSString *)name
                                     multiplicity:(PTDiffusionMultiplicity *)multiplicity;

/**
 Returns a string field metadata object initialized with the given name, given
 default value and given multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.
 
 @param defaultValue The default value to use for this field if one is not specified.

 @param multiplicity The multiplicity to use. It governs how many fields
 can be transformed using the receiver from the enclosing record.

 @exception NSInvalidArgumentException If any given argument is `nil`.

 @since 5.7.1
 */
+(PTDiffusionStringFieldMetadata *)stringWithName:(NSString *)name
                                     defaultValue:(NSString *)defaultValue
                                     multiplicity:(PTDiffusionMultiplicity *)multiplicity;

/**
 Returns a number field metadata object initialized with the given name, default multiplicity
 and default scale.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @exception NSInvalidArgumentException If the given name is `nil`.

 @since 5.6
 */
+(PTDiffusionNumberFieldMetadata *)numberWithName:(NSString *)name;

/**
 Returns a number field metadata object initialized with the given name, default scale
 and given multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @param multiplicity The multiplicity to use. It governs how many fields
 can be transformed using the receiver from the enclosing record.

 @exception NSInvalidArgumentException If either the given name or multiplicity is `nil`.

 @since 5.6
 */
+(PTDiffusionNumberFieldMetadata *)numberWithName:(NSString *)name
                                     multiplicity:(PTDiffusionMultiplicity *)multiplicity;

/**
 Returns a number field metadata object initialized with the given name, given scale
 and default multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @param scale The scale to use. It governs how many digits lie to the right
 of the decimal point. A value of zero (`0`) constrains transformed fields to integer values.

 @exception NSInvalidArgumentException If the given name is `nil`.

 @since 5.6
 */
+(PTDiffusionNumberFieldMetadata *)numberWithName:(NSString *)name
                                            scale:(NSUInteger)scale;

/**
 Returns a number field metadata object initialized with the given name, given scale
 and given multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @param multiplicity The multiplicity to use. It governs how many fields
 can be transformed using the receiver from the enclosing record.

 @param scale The scale to use. It governs how many digits lie to the right
 of the decimal point. A value of zero (`0`) constrains transformed fields to integer values.

 @exception NSInvalidArgumentException If either the given name or multiplicity is `nil`.

 @since 5.6
 */
+(PTDiffusionNumberFieldMetadata *)numberWithName:(NSString *)name
                                     multiplicity:(PTDiffusionMultiplicity *)multiplicity
                                            scale:(NSUInteger)scale;

/**
 Returns a number field metadata object initialized with the given name, given default
 value, given scale and given multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @param defaultValue The default value to use for this field if one is not specified.

 @param multiplicity The multiplicity to use. It governs how many fields
 can be transformed using the receiver from the enclosing record.

 @param scale The scale to use. It governs how many digits lie to the right
 of the decimal point. A value of zero (`0`) constrains transformed fields to integer values.

 @exception NSInvalidArgumentException If either the given name, defaultValue or
 multiplicity is `nil`.

 @since 5.7.1
 */
+(PTDiffusionNumberFieldMetadata *)numberWithName:(NSString *)name
                                     defaultValue:(NSNumber *)defaultValue
                                     multiplicity:(PTDiffusionMultiplicity *)multiplicity
                                            scale:(NSUInteger)scale;

@end

NS_ASSUME_NONNULL_END
