//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief Multiplicity describes the number of times that a metadata node may occur within its parent.
 
 Multiplicity is defined in terms of minimum count and maximum count values.

 The maximum count is boxed in a number object which enables it to be optional. Specifying
 `nil` for the maximum count indicates that there should be no limit on the maximum number
 of occurrences.
 
 Multiplicity objects are described using standard UML notation.
 
 @see PTDiffusionRecordMetadata
 @see PTDiffusionFieldMetadata

 @since 5.6
 */
@interface PTDiffusionMultiplicity : NSObject <NSCopying, NSSecureCoding>

/**
 Returns a multiplicity used for metadata nodes that have no parent (`0..0`).
 
 @since 5.7
 */
+(instancetype)multiplicityForNone;

/**
 Returns a boundless multiplicity (`0..*`).
 
 @since 5.7
 */
+(instancetype)multiplicityForUnbounded;

/**
 Returns a multiplicity specifying exactly once instance (`1`).

 @since 5.6
 */
+(instancetype)multiplicityForSingleRequired;

/**
 Returns a multiplicity specifying zero or one instance (`0..1`).

 @since 5.6
 */
+(instancetype)multiplicityForSingleOptional;

/**
 Returns a multiplicity specifying an exact number of instances (`n`).

 @param fixedCount The maximum number of instances expected which must
 be above zero.

 @since 5.6
 */
+(instancetype)multiplicityWithFixedCount:(NSUInteger)fixedCount;

/**
 Returns a multiplicity specifying a minimum number of instances (`n..*`).
 
 @param minimumCount The minimum number of instances expected.

 @since 5.6
 */
+(instancetype)multiplicityWithMinimumCount:(NSUInteger)minimumCount;

/**
 Returns a multiplicity specifying a maximum number of instances (`0..n`).
 
 @param maximumCount The maximum number of instances expected which must
 be above zero.
 
 @exception NSInvalidArgumentException If the maximum count is zero.

 @since 5.6
 */
+(instancetype)multiplicityWithMaximumCount:(NSUInteger)maximumCount;

/**
 Returns a multiplicity as described by the given string.
 
 @param multiplicity If the string could be parsed then this will contain
 the parsed multiplicity on return.
 
 @param string The string to be parsed of the form `n` or `n..m|*` where `n`
 and `m` are non-negative integers and `m` is equal to or larger than `n`.
 
 @param error In the case of failed parsing and if set to non-`nil` then this
 will contain the reason for failure on return.
 
 @exception NSInvalidArgumentException If either multiplicity or string are `nil`.
 
 @since 5.7
 */
+(BOOL)parseMultiplicity:(out PTDiffusionMultiplicity *_Nullable __strong *_Nonnull)multiplicity
              fromString:(NSString *)string
                   error:(out NSError **)error;

/**
 Returns the multiplicity for single required (`1`).

 @since 5.6
 */
+(instancetype)defaultMultiplicity;

-(instancetype)init NS_UNAVAILABLE;

/**
 Returns a multiplicity object initialized with the given minimum count and maximum count values.

 @param minimumCount The minimum number of instances expected.
 
 @param maximumCount The maximum number of instances expected. Specify `nil` for unlimited (`*`).
 If specified then must not be less than the minimum count or 1 (whichever is bigger).

 @exception NSInvalidArgumentException If the maximum count value is out of acceptable range.

 @since 5.6
 */
-(instancetype)initWithMinimumCount:(NSUInteger)minimumCount
                       maximumCount:(nullable NSNumber *)maximumCount NS_DESIGNATED_INITIALIZER;

/**
 The minimum number of instances expected.

 @since 5.6
 */
@property(nonatomic, readonly) NSUInteger minimumCount;

/**
 The maximum number of instances expected or `nil` if unlimited (`*`).

 @since 5.6
 */
@property(nonatomic, readonly, nullable) NSNumber* maximumCount;

/**
 `YES` if the maximum count value specifies an unlimited number of instances (`*`).

 @since 5.6
 */
@property(nonatomic, readonly, getter=isUnbounded) BOOL unbounded;

/**
 `YES` if the minimum count and maximum count values differ.

 @since 5.6
 */
@property(nonatomic, readonly, getter=isVariable) BOOL variable;

/**
 Compares the receiver to the given multiplicity.

 @param multiplicity The multiplicity object with which to compare the receiver.

 @return `YES` if the given multiplicity is equal to the receiver, otherwise `NO`.

 @since 5.6
 */
-(BOOL)isEqualToMultiplicity:(PTDiffusionMultiplicity *)multiplicity;

@end

NS_ASSUME_NONNULL_END
