//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionSessionErrorHandler.h>
#import <Diffusion/PTDiffusionSessionReconnectionStrategy.h>

@class PTDiffusionCredentials;

extern const NSTimeInterval PTDiffusionSessionDefaultReconnectionTimeout;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief A session configuration defines behavior and policies to use when connecting to Diffusion.
 
 @see PTDiffusionSession

 @since 5.6
 */
@interface PTDiffusionSessionConfiguration : NSObject <NSMutableCopying>

/**
 Returns a session configuration object initialized with the given principal and credentials.

 @param principal The security principal to use when opening the session.
 A value of `nil` dictates that no principal name will be associated with the session,
 indicating anonymous authentication.

 @param credentials The security credentials to use when opening the session.
 A value of `nil` dictates that no credentials will be used when authenticating the session.

 @since 5.6
 */
-(instancetype)initWithPrincipal:(nullable NSString *)principal
                     credentials:(nullable PTDiffusionCredentials *)credentials;

/**
 The security principal to use when opening the session.
 
 A value of `nil` dictates that no principal name will be associated with the session,
 indicating anonymous authentication.

 @since 5.6
 */
@property(nonatomic, nullable, readonly, copy) NSString *principal;

/**
 The security credentials to use when opening the session.
 
 A value of `nil` dictates that no credentials will be used when authenticating the session.

 @since 5.6
 */
@property(nonatomic, nullable, readonly, copy) PTDiffusionCredentials *credentials;

/**
 The error handler to be used once the session has been opened and after the open
 completion handler has been called.
 
 A value of `nil` dictates that the default error handler will be employed.

 @deprecated Since 5.9. Will be removed in a future release.
 Replaced by the `error` property on PTDiffusionSessionState.

 @since 5.6.6
 */
@property(nonatomic, nullable, readonly) id<PTDiffusionSessionErrorHandler> errorHandler __deprecated_msg("Will be removed in a future release.");

/**
 The reconnection timeout in seconds that will be used on connection failure.
 
 The reconnection timeout boxed by this number is accessed as a double representing an NSTimeInterval.
 
 A value of `nil` or a negative boxed value both dictate that reconnection will be disabled.

 @since 5.6
 */
@property(nonatomic, nullable, readonly) NSNumber* reconnectionTimeout;

/**
 The reconnection strategy that will be used on connection failure.
 
 A value of `nil` when a valid reconnectionTimeout has been set dictates that the default
 reconnection strategy will be employed.

 @since 5.6
 */
@property(nonatomic, nullable, readonly) id<PTDiffusionSessionReconnectionStrategy> reconnectionStrategy;

/**
 The default connection timeout (2 seconds).
 
 @since 5.7
 */
+(NSTimeInterval)defaultConnectionTimeout;

/**
 The connection timeout.
 
 This constrains the time taken to establish an initial connection to the server.
 The server is responsible for limiting the overall time taken to complete a connection
 once it has received the request.

 @since 5.7
 */
@property(nonatomic, readonly) NSTimeInterval connectionTimeout;

/**
 Security settings to be applied to the underlying transport streams for SSL/TLS
 encrypted connections.
 
 A value of `nil` or an empty dictionary indicates that default settings, as
 defined by the host operating system, will be applied.

 Supported keys are documented by Apple under 'CFStream Property SSL Settings
 Constants' in their Core Foundation
 [CFStream Reference](https://developer.apple.com/library/ios/documentation/CoreFoundation/Reference/CFStreamConstants/#//apple_ref/doc/constant_group/CFStream_Property_SSL_Settings_Constants).
 
 For example, a rudimentary approach to allowing otherwise disallowed self-signed
 certificates could be to disable validation of the certificate chain entirely:
 
     configuration.sslOptions =
         [NSDictionary dictionaryWithObject:kCFBooleanFalse
                                     forKey:kCFStreamSSLValidatesCertificateChain];

 @since 5.7.1
 */
@property(nonatomic, nullable, readonly, copy) NSDictionary* sslOptions;

/**
 Compares the receiver to the given session configuration.

 @param sessionConfiguration The session configuration object with which to compare the receiver.

 @return `YES` if the receiver and the given session configuration will have the same effect.

 @since 5.6
 */
-(BOOL)isEqualToSessionConfiguration:(nullable PTDiffusionSessionConfiguration *)sessionConfiguration;

@end

/**
 @brief A mutable session configuration can be modified prior to being used to define
 behaviour and policies to use when connection to Diffusion.

 @since 5.6
 */
@interface PTDiffusionMutableSessionConfiguration : PTDiffusionSessionConfiguration

/**
 The security principal to use when opening the session.

 A value of `nil` dictates that no principal name will be associated with the session,
 indicating anonymous authentication.

 @since 5.6
 */
@property(nonatomic, nullable, readwrite, copy) NSString *principal;

/**
 The security credentials to use when opening the session.

 A value of `nil` dictates that no credentials will be used when authenticating the session.

 @since 5.6
 */
@property(nonatomic, nullable, readwrite, copy) PTDiffusionCredentials *credentials;

/**
 The error handler to be used once the session has been opened and after the open
 completion handler has been called.

 A value of `nil` dictates that the default error handler will be employed. This should
 not be used for production applications as it will pollute the system log.

 @note Any failures in initially opening the session will be reported only to the open
 completion handler. The error handler supplied here will only be told about errors
 experienced once the session has been initally opened.

 @see PTDiffusionSessionDefaultErrorHandler

 @deprecated Since 5.9. Will be removed in a future release.
 Replaced by the `error` property on PTDiffusionSessionState.

 @since 5.6.6
 */
@property(nonatomic, nullable, readwrite) id<PTDiffusionSessionErrorHandler> errorHandler __deprecated_msg("Will be removed in a future release.");

/**
 The reconnection timeout in seconds that will be used on connection failure.

 The reconnection timeout boxed by this number is accessed as a double representing an NSTimeInterval.

 A value of `nil` or a negative boxed value both dictate that reconnection will be disabled.

 @since 5.6
 */
@property(nonatomic, nullable, readwrite) NSNumber* reconnectionTimeout;

/**
 The reconnection strategy that will be used on connection failure.

 A value of `nil` when a valid reconnectionTimeout has been set dictates that the default
 reconnection strategy will be employed.

 @since 5.6
 */
@property(nonatomic, nullable, readwrite) id<PTDiffusionSessionReconnectionStrategy> reconnectionStrategy;

/**
 The connection timeout.

 This constrains the time taken to establish an initial connection to the server.
 The server is responsible for limiting the overall time taken to complete a connection
 once it has received the request.

 If not explicitly set, defaultConnectionTimeout will be assumed.
 
 @note If this exceeds one hour (3,600 seconds) a warning will be logged and the
 connection timeout will be constrained to one hour.

 @since 5.7
 */
@property(nonatomic, readwrite) NSTimeInterval connectionTimeout;

/**
 Security settings to be applied to the underlying transport streams for SSL/TLS
 encrypted connections.
 
 A value of `nil` or an empty dictionary indicates that default settings, as
 defined by the host operating system, will be applied.
 
 Supported keys are documented by Apple under 'CFStream Property SSL Settings
 Constants' in their Core Foundation
 [CFStream Reference](https://developer.apple.com/library/ios/documentation/CoreFoundation/Reference/CFStreamConstants/#//apple_ref/doc/constant_group/CFStream_Property_SSL_Settings_Constants).
 
 For example, a rudimentary approach to allowing otherwise disallowed self-signed
 certificates could be to disable validation of the certificate chain entirely:
 
     configuration.sslOptions =
         [NSDictionary dictionaryWithObject:kCFBooleanFalse
                                     forKey:kCFStreamSSLValidatesCertificateChain];

 @since 5.7.1
 */
@property(nonatomic, nullable, readwrite, copy) NSDictionary* sslOptions;

@end

NS_ASSUME_NONNULL_END
